#!/usr/bin/perl -w
##############################################################################
# Prepare data about the given application to be inserted into the database.
#
# This script takes a list of components to be processed from file and
# calls 'componenttodb.pl' for each file. The results are collected in
# one file that can be used by 'upload_app_data.pl' for adding information
# to the database.
#
# Author: Denis Silakov <silakov@ispras.ru>
##############################################################################

use strict;
use Getopt::Long;
use Text::Wrap;
Getopt::Long::Configure ("posix_default", "no_ignore_case");

use Distros;
# Application information
my $AppName="";
my $AppVer="";
my $ComponentsFile;
my $RpmFile;
my $RpmFileList;
my $DebFile;
my $DebFileList;
my $AppFilesList;
my $Comment='';
my $Summary='';
my $Aarch="";
my $OutputFile;
my $Packager="";
my $URL="";
my $LicenseType="";
my $AppUI="";
my $AppFuncCategory="";

# Package manager used in the distribution
my $PackageManager;

sub HELP_MESSAGE();
sub guess_platform($ );
sub guess_rpm_appname();
sub guess_deb_appname();
sub get_app_version();
sub get_app_summary();
sub get_app_packager();
sub get_app_url();
sub get_app_license();
sub guess_deb_arch();

# options
my $opt_help;
my $quiet=1;
my $totally_quiet;
my $trsp_symlink;

GetOptions("help|h!" => \$opt_help,
            "comment|c=s" => \$Comment,
            "summary|s=s" => \$Summary,
            "category=s" => \$AppFuncCategory,
            "outfile|o=s" => \$OutputFile,
            "arch|a=s" => \$Aarch,
            "version|v=s" => \$AppVer,
            "licensetype|t=s" => \$LicenseType,
            "ui=s" => \$AppUI,
            "infile|i=s" => \$ComponentsFile,
            "rpm|r=s" => \$RpmFile,
            "rpmlist=s" => \$RpmFileList,
            "deb|d=s" => \$DebFile,
            "deblist=s" => \$DebFileList,
            "filelist|l=s" => \$AppFilesList,
            "packagemanager|m=s" => \$PackageManager,
            "packager|p=s" => \$Packager,
            "url|u=s" => \$URL,
            "verbose" => sub {$quiet=0;},
            "quiet|q!" => \$totally_quiet,
            "trsp-symlink" => sub {$trsp_symlink=1;},
            );

HELP_MESSAGE() if (defined $opt_help);

if(@ARGV != 1 and !$DebFile and !$RpmFile ) {HELP_MESSAGE(); exit(1);}

# Guess package manager, if it was not specified
if( (not $AppFilesList) and (not $PackageManager) and (not $RpmFile) and (not $RpmFileList) and (not $DebFile) and (not $DebFileList) ) {
    $PackageManager = guess_package_manager();
    if( $PackageManager eq "unknown" ) {
        die "Cannot guess package manager! Please specify it directly using '-m' option.";
    }
    else {
        warn "No package manager was specified, guessing it is $PackageManager";
    }
}

# Name should be provided as argument, but for single rpm and debs we can extract it from file
if( @ARGV == 1 ) {
    # Allow rpm/deb file name to be an argument
    if( -e ($ARGV[0]) and $ARGV[0] =~ /\.rpm$/i ) {
        $RpmFile = $ARGV[0];
        $AppName = guess_rpm_appname();
        if( !$AppName ) {
            $RpmFile = "";
        }
    }
    elsif( -e ($ARGV[0]) and $ARGV[0] =~ /\.deb$/i ) {
        $DebFile = $ARGV[0];
        $AppName = guess_deb_appname();
        if( !$AppName ) {
            $DebFile = "";
        }
    }

    if( !$AppName ) {
        $AppName = $ARGV[0];
    }
}
elsif( $DebFile ) {
    $AppName = guess_deb_appname();
}
elsif( $RpmFile ) {
    $AppName = guess_rpm_appname();
}

if( !$AppName ) {
    die "Cannot guess application name. Please specify it manually.";
}

# Check for required tools
if( ($RpmFile or $RpmFileList) and !does_shell_know("rpm") ) {
    die "You are trying to process rpm file(s), but you don't have rpm installed in your system.";
}

if( ($RpmFile or $RpmFileList) and !does_shell_know("rpm2cpio") ) {
    die "You are trying to process rpm file(s), but you don't have rpm2cpio installed in your system.";
}

if( ($DebFile or $DebFileList) and !does_shell_know("dpkg-deb") ) {
    die "You are trying to process deb file(s), but you don't have dpkg-deb installed in your system.";
}

# Try to guess some application properties if they are not provided through options
if( !$Packager and !$RpmFile and !$RpmFileList and !$DebFile and !$DebFileList ) {
    if( $AppFilesList or $ComponentsFile ) {
        warn "You should specify packager manually when you use files or components list.";
    }
    else {
        get_app_packager();
    }
}

if( !$URL ) {
    if( $AppFilesList or $ComponentsFile or $RpmFileList or $DebFileList ) {
        print STDERR "You should specify URL manually when you use files or components list.\n";
    }
    else {
        get_app_url();
    }
}

if( !$LicenseType ) {
    if( $AppFilesList or $ComponentsFile or $RpmFileList or $DebFileList ) {
        print STDERR "You should specify license type manually when you use files or components list.\n";
    }
    else {
        get_app_license();
    }
}

if( $AppUI and ($AppUI ne "GUI") and ($AppUI ne "non-GUI") ) {
    $AppUI = "";
}

if( !$AppVer ) {
    # Guess package manager, if it was not specified
    if( $AppFilesList or $ComponentsFile or $RpmFileList or $DebFileList ) {
        warn "You should specify version manually when you use files or components list.";
    }
    else {
        get_app_version();
    }
}

if( $Aarch eq 'x86' ) {
    $Aarch = 'IA32';
}

if( not $Aarch ) {
    if( $DebFile ) {
        guess_deb_arch();
    }
    if( not $Aarch ) {
        $Aarch = guess_platform(`uname -m`);
    }
}

if( !$Summary ) {
    # Application summary
    if( $AppFilesList or $RpmFileList or $DebFileList ) {
        print STDERR "If you want to add summary for you application, specify it with '-s' option.\n";
    }
    else {
        get_app_summary();
    }
}

# default output file name
if( not $OutputFile ) {
    my $outArch = lc($Aarch);
    if( $outArch =~ /ia32/ ) {
        $outArch = 'x86';
    }
    $OutputFile = $AppName."_".$AppVer."_".$outArch."_upload_data";
    $OutputFile =~ s/\//\_/g;
}

##############################################################################
# Main section
##############################################################################

system "echo \"!Application '$AppName' '$AppVer' '$Aarch' '$Packager' '$URL' '$Comment'\" > \"$OutputFile\"";
system "echo \"!AppCategory '$LicenseType' '$AppUI' '$AppFuncCategory'\" >> \"$OutputFile\"";

if( $Comment ) {
    # We'll note enclose comment with quotes - let's treat the whole string after '!AppComment' as comment text
    system "echo \"!AppComment $Comment\" >> \"$OutputFile\"";
}

my $tmpDir = "/var/tmp/apptodb_$$";

my $options = "";
if( $totally_quiet ) {
    $options.= " -q ";
}
if( !$quiet ) {
    $options.= " -verbose ";
}

# For deb and rpms - unpack contents to $tmpDir and set '-l' option for componenttodb.pl
if( $RpmFile ) {
    system "rm -rf $tmpDir; mkdir $tmpDir; cp \"$RpmFile\" $tmpDir/";
    my $cpio = $quiet ? "cpio -id --quiet" : "cpio -id";
    system "cd $tmpDir/ && rpm2cpio \"$RpmFile\" | $cpio ";
    system "rm \"$tmpDir/$RpmFile\"";
    system "echo \"$tmpDir/\" > $tmpDir/apptodb_list";
    $options.= " -l $tmpDir/apptodb_list";
}
elsif( $RpmFileList ) {
    system "rm -rf $tmpDir; mkdir $tmpDir";
    my $cpio = $quiet ? "cpio -id --quiet" : "cpio -id";

    open( RPM_LIST, "<$RpmFileList" );
    while( <RPM_LIST> )
    {
        chomp;
        if( $_ ) {
            my $base=`basename $_`;
            chomp($base);
            system "cp \"$_\" $tmpDir/ ";
            system "cd $tmpDir/ && rpm2cpio \"$base\" | $cpio ";
            system "rm \"$tmpDir/$base\"";
        }
    }
    close( RPM_LIST );

    system "echo \"$tmpDir/\" > $tmpDir/apptodb_list";
    $options.= " -l $tmpDir/apptodb_list";
}
elsif( $DebFile ) {
    system "rm -rf $tmpDir; mkdir $tmpDir; cp \"$DebFile\" $tmpDir/";
    system "cd $tmpDir/ && dpkg-deb -x \"$DebFile\" ./ ";
    system "rm \"$tmpDir/$DebFile\"";
    system "echo \"$tmpDir/\" > $tmpDir/apptodb_list";
    $options.= " -l $tmpDir/apptodb_list";
}
elsif( $DebFileList ) {
    system "rm -rf $tmpDir; mkdir $tmpDir";

    open( RPM_LIST, "<$DebFileList" );
    while( <RPM_LIST> )
    {
        chomp;
        if( $_ ) {
            my $base=`basename $_`;
            chomp($base);
            system "cp \"$_\" $tmpDir/ ";
            system "cd $tmpDir/ && dpkg-deb -x \"$base\" ./ ";
            system "rm \"$tmpDir/$base\"";
        }
    }
    close( RPM_LIST );

    system "echo \"$tmpDir/\" > $tmpDir/apptodb_list";
    $options.= " -l $tmpDir/apptodb_list";
}
elsif( $AppFilesList ) {
    $AppFilesList =~ s/\"/\\\"/g;
    $options.= " -l \"$AppFilesList\" ";
}

if( $PackageManager ) {
    $options.= " -m $PackageManager ";
}

if( $trsp_symlink ) {
    $options.= " -trsp-symlink";
}

if( $ComponentsFile ) {
    open( COMPONENT_LIST, "<$ComponentsFile" );
    while( <COMPONENT_LIST> )
    {
        chomp;
        # in the input file we can find either the component name
        # and the package that provide it or only the component name
        if( /([\w:\-\.\d]+) (.+)/) {
            my $cmd = "./componenttodb.pl $options -c \"$1\" \"$2\"";
            system( $cmd ) == 0 or die "componenttodb.pl failed: '$cmd'";

            # component name may contain slashes; componenttodb will transform them to underscores
            my $comptodb_out = $2;
            $comptodb_out =~ s/\//_/g;
            system "cat \"".$comptodb_out."_upload_data\" >> \"$OutputFile\"";
            system "rm \"".$comptodb_out."_upload_data\"";
        }
        elsif( $_ ) {
            my $cmd = "./componenttodb.pl $options \"$_\"";
            system( $cmd ) == 0 or die "componenttodb.pl failed: '$cmd'";

            my $comptodb_out = $_;
            $comptodb_out =~ s/\//_/g;
            system "cat \"".$comptodb_out."_upload_data\" >> \"$OutputFile\"";
            system "rm \"".$comptodb_out."_upload_data\"";
        }
    }
}
else {
    # No file with packages
    my $cmd = "./componenttodb.pl $options \"$AppName\"";
    system( $cmd ) == 0 or die "componenttodb.pl failed: '$cmd'";

    my $comptodb_out = $AppName;
    $comptodb_out =~ s/\//_/g;
    system "cat \"".$comptodb_out."_upload_data\" >> \"$OutputFile\"";
    system "rm \"".$comptodb_out."_upload_data\"";

    # Delete "$tmpDir" from result file if we are processing rpms or debs
    if( $RpmFile or $RpmFileList or $DebFile or $DebFileList ) {
        my $tmpFile = $OutputFile.".tmp";
        my $tmpDir_escaped = $tmpDir;
        $tmpDir_escaped =~ s/\//\\\\\//g;
        system "cat \"$OutputFile\" | sed s/$tmpDir_escaped// > \"$tmpFile\"";
        system "mv \"$tmpFile\" \"$OutputFile\"";
    }
}

if( $RpmFile or $RpmFileList or $DebFile or $DebFileList ) {
    system "rm -rf $tmpDir";
}

##############################################################################
# Miscellaneous functions - obtain different application information
##############################################################################

sub get_app_packager()
{
    my $PkgLine;

    if( $PackageManager eq "rpm" ) {
        # 'Distribution' field (if any) is usually the most informative
        $PkgLine = `rpm -q --info $AppName | LC_ALL=C grep Distribution | cut -d\\: -f2- `;

        if( !$PkgLine or $PkgLine =~ '\(none\)' ) {
            $PkgLine = `rpm -q --info $AppName | LC_ALL=C grep Vendor | cut -d\\: -f3- `;
        }

        # 'Packager' field can contain email embreced with '<>' - cut it
        if( !$PkgLine or $PkgLine =~ '\(none\)' ) {
            $PkgLine = `rpm -q --info $AppName | LC_ALL=C grep Packager | cut -d\\: -f2- | cut -d\\< -f1`;
        }

        $Packager = trim($PkgLine);
        if( !$Packager ) {
            warn "Can't detect application packager!" if( !$totally_quiet );
        }
    }
    elsif( $PackageManager eq "dpkg" ) {
        # Actually, two 'Maintainer' fields can exist - 'Maintainer' and 'Original-Maintainer'
        # The former will be printed first and this is what we need. Otherwise 'Original-Maintainer'
        # is our only choise.
        $PkgLine = `dpkg -s $AppName | LC_ALL=C grep Maintainer | cut -d\\: -f2- | cut -d\\< -f1`;
        $Packager = trim($PkgLine);
        if( !$Packager ) {
            warn "Can't detect application packager!"  if( !$totally_quiet );
        }
    }
    elsif( $PackageManager eq "emerge" ) {
        $Packager = 'Gentoo';
    }
    else {
        warn "Can't detect application packager - package manager '$PackageManager' is not supproted" if( !$totally_quiet );
    }
}

sub get_app_url()
{
    my $URLLine;

    if( $RpmFile ) {
        $URLLine = `rpm -qivp $RpmFile | LC_ALL=C grep URL | cut -d\\: -f2- `;
        $URL = trim($URLLine);
        if( !$URL ) {
            warn "Can't detect application url!" if( !$totally_quiet );
        }

        return;
    }

    if( $DebFile ) {
        $URLLine = `dpkg-deb -I \"$DebFile\" | LC_ALL=C grep Homepage `;
        if( $URLLine =~ /Homepage\s*\:\s*(.+)/ ) {
            $URL = trim($1);
            $URL =~ s/\.$//;
        }
        else {
            warn "Can't get application url!" if( !$totally_quiet );
        }

        return;
    }

    if( $PackageManager eq "rpm" ) {
        $URLLine = `rpm -q --info $AppName | LC_ALL=C grep URL | cut -d\\: -f2- `;
        $URL = trim($URLLine);
        if( !$URL ) {
            warn "Can't detect application url!" if( !$totally_quiet );
        }
    }
    elsif( $PackageManager eq "dpkg" ) {
        warn "Can't detect application url - package manager '$PackageManager' is not supproted yet" if( !$totally_quiet );
    }
    elsif( $PackageManager eq "emerge" ) {
        my $ebuild_str = "";
        $ebuild_str = `equery which $AppName`;
        if( $ebuild_str ) {
            chomp $ebuild_str;
            my $url_str = "";
            $url_str = `cat $ebuild_str | LC_ALL=C grep HOMEPAGE`;
            if( $url_str =~ /HOMEPAGE\=\"([^\"]+)\"/ ) {
                $URL = $1;
            }
            else {
                warn "Can't detect application url - invalid link in ebuild" if( !$totally_quiet );
            }
        }
        else {
            warn "Can't detect application url - something wrong with ebuild string (or maybe you don't have 'equery' installed?" if( !$totally_quiet );
        }
    }
    else {
        warn "Can't detect application url - package manager '$PackageManager' is not supproted" if( !$totally_quiet );
    }
}

# Given the license name, decide if this is an 'Open Source' license
sub guess_license_type($ )
{
    my $License = $_[0];

    if( $License =~ /GNU/ or $License =~ /GPL/ or $License =~ /BSD/ or $License =~ /MIT/ or $License =~ /MPL/
            or $License =~ /Public Domain/ or $License =~ /Apache Software License/ ) {
        $LicenseType="Open Source";
    }
}

sub guess_rpm_appname()
{
    my $NameLine;
    my $Name;

    $NameLine = `rpm -qivp \"$RpmFile\" | LC_ALL=C grep Name`;
    $Name = trim($NameLine);

    if( $Name =~ /Name\s*\:\s*(\S+)\s*Relocations\s*\:/ ) {
        $AppName = $1;
    }
}

sub guess_deb_appname()
{
    my $NameLine;
    my $Name;

    $NameLine = `dpkg-deb -I \"$DebFile\" | LC_ALL=C grep Package`;
    $Name = trim($NameLine);

    if( $Name =~ /Package\s*\:\s*(\S+)/ ) {
        $AppName = $1;
    }
}

sub guess_deb_arch()
{
    my $ArchLine;

    $ArchLine = `dpkg-deb -I \"$DebFile\" | LC_ALL=C grep Architecture`;
    $ArchLine = trim($ArchLine);

    if( $ArchLine =~ /Architecture\s*\:\s*(\S+)/ ) {
        $Aarch = guess_platform($1);
        if( $Aarch eq 'unknown' ) {
            $Aarch = '';
        }
    }
}

sub get_app_license()
{
    my $LicenseLine;
    my $License;

    if( $RpmFile ) {
        $LicenseLine = `rpm -qivp \"$RpmFile\" | LC_ALL=C grep License | cut -d\\: -f3- `;
        $License = trim($LicenseLine);
        if( !$License ) {
            warn "Can't detect application license type!" if( !$totally_quiet );
        }

        guess_license_type($License);
        return;
    }

    if( $DebFile ) {
        warn "Can't detect application license type - license auto-detection is not supported for deb packages" if( !$totally_quiet );
        return;
    }

    if( $PackageManager eq "rpm" ) {
        $LicenseLine = `rpm -q --info $AppName | LC_ALL=C grep License | cut -d\\: -f3- `;
        $License = trim($LicenseLine);
        if( !$License ) {
            warn "Can't detect application license!" if( !$totally_quiet );
            return;
        }
    }
    elsif( $PackageManager eq "dpkg" ) {
        warn "Can't detect application license - package manager '$PackageManager' is not supproted yet" if( !$totally_quiet );
        return;
    }
    elsif( $PackageManager eq "emerge" ) {
        print STDERR "Can't detect application license type - license auto-detection is not supported for emerge\n";
        return;
#       my $ebuild_str = "";
#       $ebuild_str = `equery which $AppName`;
#       if( $ebuild_str ) {
#           chomp $ebuild_str;
#           my $url_str = "";
#           $url_str = `cat $ebuild_str | LC_ALL=C grep HOMEPAGE`;
#           if( $url_str =~ /HOMEPAGE\=\"([^\"]+)\"/ ) {
#               $URL = $1;
#           }
#           else {
#               print STDERR "Can't detect application url - invalid link in ebuild\n";
#           }
#       }
#       else {
#               print STDERR "Can't detect application url - something wrong with ebuild string (or maybe you don't have 'equery' installed?\n";
#       }
    }
    else {
        warn "Can't detect application license - package manager '$PackageManager' is not supproted" if( !$totally_quiet );
    }

    guess_license_type($License);
}

sub get_app_version()
{
    my $VerLine;

    if( $RpmFile ) {
        $VerLine = `rpm -qivp \"$RpmFile\" | LC_ALL=C grep Version | LC_ALL=C grep Vendor`;
        chomp($VerLine);
        if( $VerLine =~ /Version\s*\:\s+((\w|\.|\+|\-|\~|\_|\d)+)\s+Vendor/ ) {
            $AppVer = $1;
        }
        else {
            warn "Can't detect application version!" if( !$totally_quiet );
        }

        return;
    }

    if( $DebFile ) {
        $VerLine = `dpkg-deb -I \"$DebFile\" | LC_ALL=C grep Version`;
        chomp($VerLine);
        if( $VerLine =~ /Version\s*\:\s+((\w|\.|\+|\-|\~|\_|\d)+)/ ) {
            $AppVer = $1;
        }
        else {
            warn "Can't detect application version!" if( !$totally_quiet );
        }

        return;
    }

    if( $PackageManager eq "rpm" ) {
        $VerLine = `rpm -qi $AppName | LC_ALL=C grep Version | LC_ALL=C grep Vendor`;
        chomp($VerLine);
        if( $VerLine =~ /Version\s+\:\s+((\w|\.|\-|\_|\d)+)\s+Vendor/ ) {
            $AppVer = $1;
        }
        else {
            warn "Can't detect application version!" if( !$totally_quiet );
        }
    }
    elsif( $PackageManager eq "dpkg" ) {
        $VerLine = `dpkg -s $AppName | LC_ALL=C grep -i version`;
        chomp($VerLine);
        if( $VerLine =~ /(V|v)ersion\s*:\s*(\S+)/ ) {
            $AppVer = $2;
        }
        else {
            warn "Can't detect application version!" if( !$totally_quiet );
        }
    }
    elsif( $PackageManager eq "emerge" ) {
        $VerLine = `emerge --pretend --nodeps $AppName | LC_ALL=C grep $AppName`;
        chomp($VerLine);
        if( $VerLine =~ /\[(\s|\w|\d)*\](\s|\w|\/|\d|\.|\-|\+)*\[((\s|\w|\d|\.|\-|\+)*)\]/ ) {
            $AppVer = $3;
        }
        elsif( $VerLine =~ /no\ ebuilds/ ) {
            warn "Can't detect application version - it seems that it is not installed" if( !$totally_quiet );
        }
        elsif( $VerLine =~ /$AppName(\s|\-)*((\w|\.|\d|\-|\+)+)/ ) {
            $AppVer = $2;
        }
        else {
            warn "Can't detect application version!" if( !$totally_quiet );
        }
    }
    else {
        warn "Can't detect application version - package manager '$PackageManager' is not supproted" if( !$totally_quiet );
    }
}

# Get application summary
# If we are processing rpm or deb file, we'll try to obtain information from
# the file itself (not that we can get info from rpm files on Debian-based systems and vise versa).
# Otherwise, let's ask system package manager for the package description
sub get_app_summary()
{
    my $CommentLine;

    if( $RpmFile ) {
        $CommentLine = `rpm -qivp \"$RpmFile\" | LC_ALL=C grep Summary | sed s/\\'/\\'\\^/g`;
        if( $CommentLine =~ /Summary\s*\:\s*(.+)/ ) {
            $Comment = $1;
        }
        else {
            warn "Can't get application summary!" if( !$totally_quiet );
        }
        return;
    }

    if( $DebFile ) {
        $CommentLine = `dpkg-deb -I \"$DebFile\" | LC_ALL=C grep Description | sed s/\\'/\\'\\^/g`;
        if( $CommentLine =~ /Description\s*\:\s*(.+)/ ) {
            $Comment = $1;
        }
        else {
            warn "Can't get application summary!" if( !$totally_quiet );
        }
        return;
    }

    if( $PackageManager eq "rpm" ) {
        $CommentLine = `rpm -qi $AppName | LC_ALL=C grep Summary | sed s/\\'/\\'\\^/g`;
        if( $CommentLine =~ /Summary\s*\:\s*(.+)/ ) {
            $Comment = $1;
        }
        else {
            warn "Can't get application summary!" if( !$totally_quiet );
        }
    }
    elsif( $PackageManager eq "dpkg" ) {
        $CommentLine = `dpkg -s $AppName | LC_ALL=C grep Description | sed s/\\'/\\'\\^/g`;
        if( $CommentLine =~ /Description\s*\:\s*(.+)/ ) {
            $Comment = $1;
        }
        else {
            warn "Can't get application summary!" if( !$totally_quiet );
        }
    }
    elsif( $PackageManager eq "emerge" ) {
        if( does_shell_know("equery") ) {
            my $ebuild_str = "";
            $ebuild_str = `equery which $AppName`;
            if( $ebuild_str ) {
                chomp $ebuild_str;
                my $comment_str = "";
                $comment_str = `cat $ebuild_str | LC_ALL=C grep DESCRIPTION | sed s/\\'/\\'\\^/g`;
                if( $comment_str =~ /DESCRIPTION\=\"([^\"]+)\"/ ) {
                    $Comment = $1;
                }
                else {
                    warn "Can't detect application url - invalid link in ebuild" if( !$totally_quiet );
                }
            }
            else {
                warn "Can't get application summary - something wrong with ebuild name" if( !$totally_quiet );
            }
        }
        else {
            warn "Can't get application summary - 'equery' tool from 'gentoolkit' package is required." if( !$totally_quiet );
        }
    }
    else {
        warn "Can't get application summary - package manager '$PackageManager' is not supproted" if( !$totally_quiet );
    }
}

sub HELP_MESSAGE()
{
print STDERR <<"EOM"

$0: Obtain information about application.

Usage: $0 [options] app_name

app_name                      Name of the application. Can be absent if single rpm or deb file is processed.

-a|--arch                     Specify architecture. Possible values are x86, IA64, PPC32,
                              PPC64, S390, S390X and x86-64. By default the output of `uname -m`
                              is analyzied

-h|--help                     Display this help message

-v|--version app_version      Application version. If none is specified, the script will try to obtain it from package manager

-i|--infile filename          File with the list of packages to be processed (i.e. packages that form the application)
                              If no file is specified, application name is considered to be the only package

-l|--filelist filename        File containing list of file names that are installed by the application.
                              (Directory names are also allowed; all files from given directories will be processed)

-r|--rpm filename             RPM file with application to be processed. Use this option if you don't want to
                              install the application and the whole application is provided by a single rpm file.
                              NOTE: RPM file can be given as script argument, without these options. In this case
                              application name will be taken from RPM information.

--rpmlist filename            List of RPM files containing application binaries. Use this option if you don't want to
                              install the application and the application is provided by several rpm files.

-d|--deb filename             Deb file with application to be processed. Use this option if you don't want to
                              install the application and the whole application is provided by a single deb file.
                              NOTE: deb file can be given as script argument, without these options. In this case
                              application name will be taken from deb information.

--deblist filename            List of Deb files containing application binaries. Use this option if you don't want to
                              install the application and the application is provided by several deb files.

-o|--outfile filename         Filename where the data for upload will be situated
                              By default, "<app_name>_<app_ver>_<arch>_upload_data" is used

-q|--quiet                    Be really quiet - do not produce any messages except critcal errors

--verbose                     Be verbose

-u|--url URL                  Application homepage URL

-t|--licensetype type         License type. "Open Source", "Proprietary" and "Mixed"  are valid values at the moment.

-p|--packager packager        Application packager - any information about uploaded application build
                              (distribution, maintainer, etc.). This can be useful since different builds of the same
                              application may require different sets of external interfaces. If nothing is specified,
                              then the script will try to guess it using package manager

-m|--packagemanager pkg_name  Package manager used in the distribution. Supported values
                              are 'rpm', 'dpkg' and 'emerge'. If none is specified, the script
                              will try to guess it

--ui ui_kind                  User interface style. Possible values are 'non-GUI' and 'GUI'. Applications that have both
                              graphical and command line interface are treated as 'GUI'

--category category_kind      Application functional category. Possible values are:
                                  'Accessibility and i18n'
                                  'Antivirus and Security'
                                  'Data Management'
                                  'Development'
                                  'Emulators'
                                  'Games'
                                  'Multimedia and Graphics'
                                  'Network'
                                  'Office and Desktop'
                                  'Science and Education'
                                  'System Tools'
                                  'X11 Utilities'

-s|--summary summary_text     Application summary. If none is specified, the script will try to obtain it from
                              package manager; if rpm or deb file is being processed, the script will try to
                              obtain necessary data from the file itself

-c|--comment comment_text     Comment for the application. Any useful information about application build process
                              can be added here (e.g. which libraries were linked statically).

EOM
      ; exit(1);
}
